@@ -297,6 +297,20 @@ class TestConstraints(BaseTest):
         with self.assertRaises(ValueError):
             con = PowCone3D(x, y, z, -0.00001)
 
+    def test_pow3d_scalar_alpha_constraint(self) -> None:
+        """
+        Simple test case with scalar AND vector `alpha`
+        inputs to `PowCone3D`
+        """""
+        x_0 = cp.Variable(shape=(3,))
+        x = cp.Variable(shape=(3,))
+        cons = [cp.PowCone3D(x_0[0], x_0[1], x_0[2], 0.25),
+                x <= -10]
+        obj = cp.Minimize(cp.norm(x - x_0))
+        prob = cp.Problem(obj, cons)
+        prob.solve()
+        self.assertAlmostEqual(prob.value, 17.320508075380552)
+
     def test_pownd_constraint(self) -> None:
         n = 4
         W, z = Variable(n), Variable()
@@ -340,38 +354,43 @@ class TestConstraints(BaseTest):
             (self.z <= self.x).__bool__()
         self.assertEqual(str(cm.exception), error_str)
 
-    def test_nonpos(self) -> None:
-        """Tests the NonPos constraint for correctness.
+    def test_nonneg(self) -> None:
+        """Solve a trivial NonNeg-constrained problem through
+        the conic and QP code paths.
         """
-        n = 3
-        x = cp.Variable(n)
-        c = np.arange(n)
-        prob = cp.Problem(cp.Maximize(cp.sum(x)),
-                          [cp.NonPos(x - c)])
-        # Solve through cone program path.
+        x = cp.Variable(3)
+        c = np.arange(3)
+        prob = cp.Problem(cp.Minimize(cp.sum(x)),
+                          [cp.NonNeg(x - c)])
         prob.solve(solver=cp.ECOS)
         self.assertItemsAlmostEqual(x.value, c)
-
-        # Solve through QP path.
         prob.solve(solver=cp.OSQP)
         self.assertItemsAlmostEqual(x.value, c)
 
-    def test_nonpos_dual(self) -> None:
-        """Test dual variables work for NonPos.
+    def test_nonpos(self) -> None:
+        """Tests the NonPos constraint for correctness with conic and
+        QP code paths.
         """
-        n = 3
-        x = cp.Variable(n)
-        c = np.arange(n)
-        prob = cp.Problem(cp.Maximize(cp.sum(x)),
-                          [(x - c) <= 0])
+        x = cp.Variable(3)
+        c = np.arange(3)
+        prob = cp.Problem(cp.Maximize(cp.sum(x)), [cp.NonPos(x - c)])
+        prob.solve(solver=cp.ECOS)
+        self.assertItemsAlmostEqual(x.value, c)
+        prob.solve(solver=cp.OSQP)
+        self.assertItemsAlmostEqual(x.value, c)
+
+    def test_nonneg_dual(self) -> None:
+        # Compute reference solution with an Inequality constraint.
+        x = cp.Variable(3)
+        c = np.arange(3)
+        objective = cp.Minimize(cp.sum(x))
+        prob = cp.Problem(objective, [c - x <= 0])
         prob.solve(solver=cp.ECOS)
         dual = prob.constraints[0].dual_value
-        prob = cp.Problem(cp.Maximize(cp.sum(x)),
-                          [cp.NonPos(x - c)])
-        # Solve through cone program path.
+        # reported dual variables are the same with NonNeg, even though
+        # the convention for how they add to the Lagrangian differs by sign.
+        prob = cp.Problem(objective, [cp.NonNeg(x - c)])
         prob.solve(solver=cp.ECOS)
         self.assertItemsAlmostEqual(prob.constraints[0].dual_value, dual)
-
-        # Solve through QP path.
         prob.solve(solver=cp.OSQP)
         self.assertItemsAlmostEqual(prob.constraints[0].dual_value, dual)
