@@ -24,7 +24,14 @@ import cvxpy.interface.matrix_utilities as intf
 import cvxpy.settings as s
 from cvxpy import Minimize, Problem
 from cvxpy.atoms.affine.add_expr import AddExpression
-from cvxpy.atoms.affine.wraps import psd_wrap
+from cvxpy.atoms.affine.wraps import (
+    hermitian_wrap,
+    nonneg_wrap,
+    nonpos_wrap,
+    psd_wrap,
+    skew_symmetric_wrap,
+    symmetric_wrap,
+)
 from cvxpy.expressions.constants import Constant, Parameter
 from cvxpy.expressions.variable import Variable
 from cvxpy.tests.base_test import BaseTest
@@ -61,8 +68,10 @@ class TestExpressions(BaseTest):
         # self.assertEqual(x.canonical_form[0].shape, (2, 1))
         # self.assertEqual(x.canonical_form[1], [])
 
-        self.assertEqual(repr(self.x), "Variable((2,))")
-        self.assertEqual(repr(self.A), "Variable((2, 2))")
+        self.assertEqual(repr(self.x), "Variable((2,), x)")
+        self.assertEqual(repr(self.A), "Variable((2, 2), A)")
+        self.assertEqual(repr(cp.Variable(name='x', nonneg=True)), "Variable((), x, nonneg=True)")
+        self.assertTrue(repr(cp.Variable()).startswith("Variable((), var"))
 
         # Test shape provided as list instead of tuple
         self.assertEqual(cp.Variable(shape=[2], integer=True).shape, (2,))
@@ -1450,13 +1459,47 @@ class TestExpressions(BaseTest):
 
     def test_matmul_scalars(self) -> None:
         """Test evaluating a matmul that reduces one argument internally to a scalar.
-
-        See https://github.com/cvxpy/cvxpy/issues/2065
         """
         x = cp.Variable((2,))
         quad = cp.quad_form(x, np.eye(2))
         a = np.array([2])
-        # NOTE quad has dimensions (1, 1) which is a bug.
-        expr = a @ quad
+        expr = quad * a
         x.value = np.array([1, 2])
-        self.assertAlmostEqual(expr.value, 10)
+        P = np.eye(2)
+        true_val = (np.transpose(x.value) @ P @ x.value) * a
+        assert quad.shape == ()
+        self.assertEqual(expr.value, true_val)
+
+    def test_wraps(self) -> None:
+        """Test wrap classes."""
+        x = cp.Variable(2)
+        expr = nonneg_wrap(x)
+        assert expr.is_nonneg()
+
+        expr = nonpos_wrap(x)
+        assert expr.is_nonpos()
+
+        Z = cp.Variable((2, 2))
+        U = cp.Variable((2, 2), complex=True)
+        expr = psd_wrap(Z)
+        assert expr.is_psd()
+        assert not expr.is_complex()
+        assert expr.is_symmetric()
+        assert expr.is_hermitian()
+
+        expr = psd_wrap(U)
+        assert expr.is_psd()
+        assert expr.is_complex()
+        assert not expr.is_symmetric()
+        assert expr.is_hermitian()
+
+        expr = symmetric_wrap(Z)
+        assert expr.is_symmetric()
+        assert expr.is_hermitian()
+
+        expr = skew_symmetric_wrap(Z)
+        assert expr.is_skew_symmetric()
+
+        expr = hermitian_wrap(U)
+        assert expr.is_hermitian()
+
