@@ -21,8 +21,17 @@ from scipy.linalg import lstsq
 
 import cvxpy as cp
 from cvxpy import Maximize, Minimize, Parameter, Problem
-from cvxpy.atoms import (QuadForm, abs, huber, matrix_frac, norm, power,
-                         quad_over_lin, sum, sum_squares,)
+from cvxpy.atoms import (
+    QuadForm,
+    abs,
+    huber,
+    matrix_frac,
+    norm,
+    power,
+    quad_over_lin,
+    sum,
+    sum_squares,
+)
 from cvxpy.expressions.variable import Variable
 from cvxpy.reductions.solvers.defines import INSTALLED_SOLVERS, QP_SOLVERS
 from cvxpy.tests.base_test import BaseTest
